
;Unicode True

!define NAME "Bible Analyzer 5.5"
!define UNINSTKEY "${NAME}" ; Using a GUID here is not a bad idea
!define DEFAULTNORMALDESTINATON "$PROGRAMFILES\Bible Analyzer 5"
!define DEFAULTPORTABLEDESTINATON "$DESKTOP\Bible Analyzer 5 Portable"

Name "${NAME}"
Outfile "BibleAnalyzer550-setup.exe"

;SetCompressor /SOLID lzma

Var NormalDestDir
Var PortableDestDir
Var PortableMode

!include LogicLib.nsh
!include FileFunc.nsh
!include MUI2.nsh
!include FontReg.nsh
!include FontName.nsh
!include WinMessages.nsh

RequestExecutionLevel admin

Function RequireAdmin
	UserInfo::GetAccountType
	Pop $8
	${If} $8 != "Admin"
		MessageBox MB_ICONSTOP "You must be logged on as an adminstrator to install ${NAME}"
		SetErrorLevel 740 ;ERROR_ELEVATION_REQUIRED
		Abort
	${EndIf}
FunctionEnd


!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_LICENSE "License.txt"
Page Custom PortableModePageCreate PortableModePageLeave
!insertmacro MUI_PAGE_DIRECTORY
!insertmacro MUI_PAGE_INSTFILES

!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES
!insertmacro MUI_LANGUAGE "English"



Function .onInit
	StrCpy $NormalDestDir "${DEFAULTNORMALDESTINATON}"
	StrCpy $PortableDestDir "${DEFAULTPORTABLEDESTINATON}"

	${GetParameters} $9

	ClearErrors
	${GetOptions} $9 "/?" $8
	${IfNot} ${Errors}
		MessageBox MB_ICONINFORMATION|MB_SETFOREGROUND "\
		  /PORTABLE : Extract application to USB drive etc$\n\
		  /S : Silent install$\n\
		  /D=%directory% : Specify destination directory$\n"
		Quit
	${EndIf}

	ClearErrors
	${GetOptions} $9 "/PORTABLE" $8
	${IfNot} ${Errors}
		StrCpy $PortableMode 1
		StrCpy $0 $PortableDestDir
	${Else}
		StrCpy $PortableMode 0
		StrCpy $0 $NormalDestDir
		${If} ${Silent}
			Call RequireAdmin
		${EndIf}
	${EndIf}

	${If} $InstDir == ""
		; User did not use /D to specify a directory, 
		; we need to set a default based on the install mode
		StrCpy $InstDir $0
	${EndIf}
	Call SetModeDestinationFromInstdir
FunctionEnd



Function SetModeDestinationFromInstdir
	${If} $PortableMode = 0
		StrCpy $NormalDestDir $InstDir
	${Else}
		StrCpy $PortableDestDir $InstDir
	${EndIf}
FunctionEnd


Function PortableModePageCreate
	Call SetModeDestinationFromInstdir ; If the user clicks BACK on the directory page we will remember their mode specific directory
	!insertmacro MUI_HEADER_TEXT "Install Mode" "Choose how you want to install ${NAME}."
	nsDialogs::Create 1018
	Pop $0
	${NSD_CreateLabel} 0 10u 100% 24u "Select installation mode:"
	Pop $0
	${NSD_CreateRadioButton} 30u 50u -30u 8u "Normal Installation"
	Pop $1
	${NSD_CreateRadioButton} 30u 70u -30u 8u "Portable (All files under one folder)"
	Pop $2
	${If} $PortableMode = 0
		SendMessage $1 ${BM_SETCHECK} ${BST_CHECKED} 0
	${Else}
		SendMessage $2 ${BM_SETCHECK} ${BST_CHECKED} 0
	${EndIf}
	nsDialogs::Show
FunctionEnd


Function PortableModePageLeave
	${NSD_GetState} $1 $0
	${If} $0 <> ${BST_UNCHECKED}
		StrCpy $PortableMode 0
		StrCpy $InstDir $NormalDestDir
		Call RequireAdmin
	${Else}
		StrCpy $PortableMode 1
		StrCpy $InstDir $PortableDestDir
	${EndIf}
FunctionEnd


Function InstallFonts
	StrCpy $FONT_DIR $FONTS
	DetailPrint "In fonts..."
	!insertmacro InstallTTFFont "GalaxieUnicodeGreek.ttf"
	!insertmacro InstallTTFFont "GalaxieUnicodeHebrew.ttf"
	;!insertmacro InstallTTFFont "semear.ttf"
	;!insertmacro InstallTTFFont "semlate.ttf"
	;!insertmacro InstallTTFFont "semmid.ttf"
	;!insertmacro InstallTTFFont "semmod.ttf"
	DetailPrint "Finished fonts..."
	SendMessage ${HWND_BROADCAST} ${WM_FONTCHANGE} 0 0 /TIMEOUT=5000
FunctionEnd


Section
SetOutPath $INSTDIR

${If} $PortableMode = 0

	;All users
	SetShellVarContext all
	
	Call InstallFonts
	
	SetOutPath $INSTDIR
	
	File *.dll
	File BibleAnalyzer5.exe
	File license.txt
	File manual.chm
	File BA-Reset.exe
	File filters.dat

	SetOutPath "$APPDATA\Bible Analyzer\Modules"
	File /r Modules\*.*  

	SetOutPath "$APPDATA\Bible Analyzer"
	File quotes.txt
	File contexts.dat
	File strongWds.dat
	
	# Make the directory "$APPDATA\Bible Analyzer" read write accessible by all users
	AccessControl::GrantOnFile "$APPDATA\Bible Analyzer" "(BU)" "GenericRead + GenericWrite"
	
	SetOutPath $INSTDIR

	;Store installation folder
	WriteRegStr HKCU "Software\BibleAnalyzer5" "" $INSTDIR  

	; Write the uninstall keys for Windows
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BibleAnalyzer5" "DisplayName" "Bible Analyzer 5"

	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BibleAnalyzer5" "DisplayIcon" '"$INSTDIR\BibleAnalyzer5.exe"'
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BibleAnalyzer5" "Publisher" "BibleAnalyzer.com"
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BibleAnalyzer5" "URLInfoAbout" "http://www.bibleanalyzer.com"
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BibleAnalyzer5" "HelpLink" "http://www.bibleanalyzer.com"
	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BibleAnalyzer5" "DisplayVersion" "5.4.1"
	WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BibleAnalyzer5" "EstimatedSize" "48000"

	WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BibleAnalyzer5" "UninstallString" '"$INSTDIR\uninstall.exe"'
	WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BibleAnalyzer5" "NoModify" 1
	WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BibleAnalyzer5" "NoRepair" 0
	WriteUninstaller "uninstall.exe"

	; Write the url protocol keys
	WriteRegStr HKCR "bibleanalyzer" "" "URL:bibleanalyzer Protocol"
	WriteRegStr HKCR "bibleanalyzer" "URL Protocol" ""
	WriteRegStr HKCR "bibleanalyzer\DefaultIcon" "" "$INSTDIR\BibleAnalyzer5.exe"
	WriteRegStr HKCR "bibleanalyzer\shell\open\command" "" '$INSTDIR\BibleAnalyzer5.exe "%1"'	

	; Shortcuts
	CreateDirectory "$SMPROGRAMS\Bible Analyzer 5"
	CreateShortCut "$SMPROGRAMS\Bible Analyzer 5\Uninstall Bible Analyzer 5.lnk" "$INSTDIR\uninstall.exe" "" "$INSTDIR\uninstall.exe" 0
	CreateShortCut "$SMPROGRAMS\Bible Analyzer 5\Bible Analyzer 5.lnk" "$INSTDIR\BibleAnalyzer5.exe" "" "$INSTDIR\BibleAnalyzer5.exe" 0
	CreateShortCut "$SMPROGRAMS\Bible Analyzer 5\Bible Analyzer Reset.lnk" "$INSTDIR\BA-Reset.exe" "" "$INSTDIR\BA-Reset.exe" 0
	CreateShortCut "$SMPROGRAMS\Bible Analyzer 5\Bible Analyzer User Manual.lnk" "$INSTDIR\manual.chm" "" "$INSTDIR\manual.chm" 0

	CreateShortCut "$DESKTOP\Bible Analyzer 5.lnk" "$INSTDIR\BibleAnalyzer5.exe" "" "$INSTDIR\BibleAnalyzer5.exe" 0
	
	
${Else}
    ; Create the file the application uses to detect portable mode
    FileOpen $0 "$INSTDIR\portable.ini" w
    FileWrite $0 "PORTABLE MODE"
    FileClose $0
	
	File GalaxieUnicodeGreek.ttf
	File GalaxieUnicodeHebrew.ttf
	;File semear.ttf
	;File semlate.ttf
	;File semmid.ttf
	;File semmod.ttf
	
	File BibleAnalyzer5.exe
	File license.txt
	File manual.chm
	File quotes.txt
	File contexts.dat
	File filters.dat
	File strongWds.dat

	SetOutPath $INSTDIR\Modules  
	File /r Modules\*.*  

	MessageBox MB_OK "Copy installation folder to portable drive and open 'BibleAnalyzer5.exe.'" 
	
	MessageBox MB_OK "Some True-Type fonts have been copied to the installation folder and should be installed on the host system for optimal Hebrew and Greek text display." 
${EndIf}
SectionEnd


Section Uninstall
	SetShellVarContext all

	; Remove registry keys
	DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BibleAnalyzer5"
	DeleteRegKey /ifempty HKCU "Software\BibleAnalyzer5"
	DeleteRegKey HKCR "bibleanalyzer"

	; Remove files and uninstaller
	Delete $INSTDIR\*.exe
	Delete $INSTDIR\*.dll
	Delete $INSTDIR\*.txt
	Delete $INSTDIR\*.ini
	Delete $INSTDIR\*.chm
	Delete $INSTDIR\*.dat

	MessageBox MB_YESNO "Delete all Bible Analyzer modules? Click 'No' if reinstalling or updating." IDNO nodelete
	RMDir /r "$APPDATA\Bible Analyzer"

	nodelete:

	SetOutPath $PROGRAMFILES

	RMDir "$INSTDIR"

	; Remove shortcuts, if any
	Delete "$SMPROGRAMS\Bible Analyzer 5\*.lnk"
	Delete "$DESKTOP\Bible Analyzer 5.lnk"
	RMDir  "$SMPROGRAMS\Bible Analyzer 5"
SectionEnd